==> glexec-0.8.10.tar.gz: glexec-0.8.10/src/glexec_lcas.c <==
/**
 * Copyright (c) Members of the EGEE Collaboration. 2010. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 * Authors: Oscar Koeroo, Mischa Sall\'e
 *      NIKHEF Amsterdam, the Netherlands
 *      <grid-mw-security@nikhef.nl>  
 */

/*
 * glexec_lcas provides the functionality to dynamically open lcas and its API
 */

#include "glexec.h"
#include "glexec_conf.h"
#include "glexec_log.h"
#include "glexec_environ.h"
#include "glexec_lcas.h"

#include "fileutil/fileutil.h"

#include <dlfcn.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

/*************************************************************************/
/* TYPEDEFS			                                         */
/*************************************************************************/

/* Either include lcas_types.h or just takeover the definition of the
 * lcas_request_t
 */
#if 0
#include "lcas_types.h"
#else
typedef char *lcas_request_t;
#endif

/* Struct containing all the lcas functions, will be filled using dlsym() */
typedef struct {
    void *handle; /* handle to the actual LCAS library */
    int (*lcas_init)(FILE*);
    int (*lcas_pem)(char*,lcas_request_t);
    int (*lcas_term)(void);
} lcas_library_t;

/*************************************************************************/
/* INTERNAL FUNCTION PROTOTYPES                                          */
/*************************************************************************/

/*
 * Loads the LCAS library and its functions using dlopen() and dlsym()
 * returns 0 on success and -1 on error.
 */
int lcas_load(const char *libdir, lcas_library_t *lcas);

/*
 * Closes the LCAS library.
 * returns 0 on success and -1 on error.
 */
int lcas_close(lcas_library_t *lcas);

/*
 * Setup LCAS environment variables
 * set libdir (might be "") where to find lcas.so
 */
void lcas_env_setup(char *logstring, const char **libdir);

/*
 * Open LCAS logging.
 * returns pointer to glexec_lcas_log file (might be NULL)
 */
FILE *lcas_log_setup(void);

/*************************************************************************/
/* FUNCTIONS			                                         */
/*************************************************************************/

/*
 * Setup, run and close LCAS for given rsl.
 * returns GLEXEC_SUCCESS when successful
 */
int glexec_lcas(char *pem_string, char *logstring, char *rsl)   {
    const char *lcas_libdir;
    FILE *lcas_log;
    lcas_library_t lcas;
    lcas_request_t request=(lcas_request_t)rsl;
    int rc;

    /* Setup LCAS environment */
    lcas_env_setup(logstring, &lcas_libdir);

    /* Load LCAS library */
    if (lcas_load(lcas_libdir,&lcas))
	return GLEXEC_LCAS_INIT_FAIL;

    /* Setup LCAS log */
    lcas_log=lcas_log_setup();
    
    /* Init will return 0 on success or otherwise on failure, note that
     * lcas_log can be NULL in which case syslog is used. */
    if (lcas.lcas_init(lcas_log))    {
	rc=GLEXEC_LCAS_INIT_FAIL;
	goto lcas_finalize_noterm;
    }

    glexec_log(GLEXEC_LOG_DEBUG, "Initialisation of LCAS has finished.\n");

    /* LCAS AuthZ call */
    rc=(lcas.lcas_pem(pem_string, request)==0 
	    ? GLEXEC_SUCCESS
	    : GLEXEC_LCAS_FAILED);

    /* terminate LCAS */
    lcas.lcas_term();

lcas_finalize_noterm:
    /* close the library */
    lcas_close(&lcas);

    /* Close the log file only if one is actually open (like in syslog usage
     * mode) */
    if (lcas_log) {
	if (fclose(lcas_log))
	    glexec_log(GLEXEC_LOG_ERROR,
		    "Error while closing lcas log file: %s\n",
		    strerror(errno));
	lcas_log = NULL;
    }

    /* Now check whether LCAS succeeded */
    if (rc==GLEXEC_SUCCESS)
	glexec_log(GLEXEC_LOG_DEBUG, "LCAS is finished\n");
    
    return rc;
}

/*
 * Print the known LCAS defines to file out.
 */
void glexec_lcas_defines(FILE *out)	{
#ifdef LCAS_DB_FILE
    fprintf(out," -D LCAS_DB_FILE=\"%s\"\n", LCAS_DB_FILE);
#endif
#ifdef LCAS_LOG_FILE
    fprintf(out," -D LCAS_LOG_FILE=\"%s\"\n", LCAS_LOG_FILE);
#endif
#ifdef LCAS_DEBUG_LEVEL
    fprintf(out," -D LCAS_DEBUG_LEVEL=\"%s\"\n", LCAS_DEBUG_LEVEL);
#endif
#ifdef LCAS_LIBDIR
    fprintf(out," -D LCAS_LIBDIR=\"%s\"\n", LCAS_LIBDIR);
#endif
}

/*************************************************************************/
/* INTERNAL FUNCTIONS			                                 */
/*************************************************************************/

/*
 * Loads the LCAS library and its functions using dlopen() and dlsym()
 * returns 0 on success and -1 on error.
 */
int lcas_load(const char *libdir, lcas_library_t *lcas)  {
    char *error=dlerror(); /* proper way to check dlsym() failed is using
			      dlerror(), see dlsym(3) */
    const char *lcas_so="liblcas" LIBSUFF;
    char lcas_lib_path[AP_MAXPATH];
    const char *lcas_lib;


    /* prefix lcmaps library name with libdir when needed */
    if (strlen(libdir)>0)   {
	if (snprintf(lcas_lib_path,AP_MAXPATH,"%s/%s",libdir,lcas_so)
		>=AP_MAXPATH)    {
	    glexec_log(GLEXEC_LOG_ERROR,"Full path %s/%s is too long\n",
		    libdir,lcas_so);
	    return -1;
	}
	lcas_lib=lcas_lib_path;
    } else
	lcas_lib=lcas_so;

    /* Load the library: note need RTLD_GLOBAL to provide its symbols to the
     * plugins! This also means they stay around after a dlclose() ! */
    glexec_log(GLEXEC_LOG_DEBUG, "Opening LCAS lib: %s\n",lcas_lib);
    if ( (lcas->handle=
		dlopen(lcas_lib,RTLD_NOW | RTLD_GLOBAL))==NULL )	{
	glexec_log(GLEXEC_LOG_ERROR,
		   "Error opening LCAS lib: %s\n",dlerror());
	return -1;
    }

    /* Load necessary basic LCAS functions */
    glexec_log(GLEXEC_LOG_DEBUG, "Loading LCAS symbols.\n");
    lcas->lcas_init=(int (*)(FILE*))dlsym(lcas->handle,"lcas_init");
    if ((error=dlerror())!=NULL)
	goto lcas_dlopen_failed;
    lcas->lcas_pem=
	(int (*)(char*,lcas_request_t))dlsym(lcas->handle,"lcas_pem");
    if ((error=dlerror())!=NULL)
	goto lcas_dlopen_failed;
    lcas->lcas_term=(int (*)(void))dlsym(lcas->handle,"lcas_term");
    if ((error=dlerror())!=NULL)
	goto lcas_dlopen_failed;
    return 0;
    
lcas_dlopen_failed:
    glexec_log(GLEXEC_LOG_ERROR,"Error loading LCAS function: %s\n",error);
    dlclose(lcas->handle);
    lcas->handle=NULL;
    return -1;
}

/*
 * Closes the LCAS library.
 * returns 0 on success and -1 on error.
 */
int lcas_close(lcas_library_t *lcas)	{
    int rc;
    glexec_log(GLEXEC_LOG_DEBUG,"Closing LCAS library.\n");
    if ((rc=dlclose(lcas->handle)))
	glexec_log(GLEXEC_LOG_ERROR,"Error closing LCAS library: %s\n",
		dlerror());
    lcas->handle=NULL;
    return (rc ? -1 : 0);
}

/*
 * Setup LCAS environment variables
 * set libdir (might be "") where to find lcas.so
 */
void lcas_env_setup(char *logstring, const char **libdir) {
    const char *moddir_suffix="/modules";
    char modules_dir[AP_MAXPATH];
    struct stat st;

    /* Get LCAS variables */
    setenv("LCAS_LOG_STRING", logstring , 1);

    glexec_setenv_regex("LCAS_DEBUG_LEVEL",
	glexec_conf_value_default("glexec:lcas_debug_level",LCAS_DEBUG_LEVEL),
	"^[0-5]$");
    glexec_setenv_regex("LCAS_DB_FILE",
	glexec_conf_value_default("glexec:lcas_db_file",LCAS_DB_FILE),
	"\\/(.*\\/)+[^/]*");

    /* Setup LCAS libdir, will be used for dlopen-ing LCAS */
    *libdir=glexec_conf_value_default("glexec:lcas_libdir",LCAS_LIBDIR);

    /* If it's non-empty, check it is an absolute and valid directory */
    if ( (*libdir)[0]!='\0')   {
	if ( (*libdir)[0]!='/' || stat(*libdir,&st)!=0 || !S_ISDIR(st.st_mode)) 
	{
	    glexec_log(GLEXEC_LOG_WARNING,
		"Warning: ignoring lcas_libdir config option: "\
		"%s is not an absolute path to a valid directory.\n", *libdir);
	    *libdir=LCAS_LIBDIR;
	}
    }
    setenv("LCAS_LIBDIR", *libdir, 1);

    /* If it's non-empty, also setup LCAS_MODULES_DIR. */
    if ( (*libdir)[0]!='\0')    { /* No need for strlen... */
	if (snprintf(modules_dir,AP_MAXPATH,"%s%s",
		     *libdir, moddir_suffix) >= AP_MAXPATH)
	    glexec_log(GLEXEC_LOG_WARNING,
		"Full modulespath '%s%s' would be too long, " \
		"not setting LCAS_MODULES_DIR\n",
		libdir,moddir_suffix);
	else	{
	    glexec_log(GLEXEC_LOG_DEBUG,"Setting LCAS_MODULES_DIR to '%s'\n",
		       modules_dir);
	    setenv("LCAS_MODULES_DIR", modules_dir, 1);
	}
    }
}

/*
 * Open LCAS logging.
 * returns pointer to glexec_lcas_log file (might be NULL)
 */
FILE *lcas_log_setup(void) {
    const mode_t file_mask=GLEXEC_LOG_FILEMASK; /* File permissions */
    const mode_t dir_mask=GLEXEC_LOG_DIRMASK; /* Dir permissions */
    int rc;
    gid_t oldgid=getegid(),lcas_loggid;
    char *lcas_log_name;
    FILE *lcas_log=NULL;

    /* Optionally open logfile */
    if (glexec_get_log_to_syslog()) { /* syslog */
	/*  Make sure the environment variables are never set when glexec has to
	 *  use syslog.*/
	unsetenv("LCAS_LOG_FILE");
    } else { /* logfile */
	glexec_setenv_regex("LCAS_LOG_FILE",
	    glexec_conf_value_default("glexec:lcas_log_file",LCAS_LOG_FILE),
		"\\/(.*\\/)+[^/]*");
	if ( (lcas_log_name = getenv("LCAS_LOG_FILE"))==NULL )	{
	    glexec_log(GLEXEC_LOG_ERROR,
		"Log file for LCAS could not be opened: "\
		"No log file name was specified.\n");
	    glexec_exit (GLEXEC_LCAS_INIT_FAIL, GLEXEC_NO_CHILD_EXIT_CODE);
	}
	/* Get logging GID as set by glexec */
	lcas_loggid=glexec_get_loggid();
	/* Set the effective gid, so that file and dir will be created with the
	 * right ownership */
	if (setegid(lcas_loggid))
	    glexec_log(GLEXEC_LOG_WARNING,
		"Setting effective gid to %d failed.\n", (int)lcas_loggid);
	/* Now try to open the logfile with file permission file_mask and directory
	 * permissions dir_mask. Don't force it to be trusted. */
	rc=cgul_open_logfile(lcas_log_name, file_mask, dir_mask, &lcas_log, 0);
	/* reset effective gid */
	if (setegid(oldgid))
	    glexec_log(GLEXEC_LOG_WARNING,
		"Setting effective gid to %d failed.\n", (int)oldgid);
	/* Parse return code of cgul_open_logfile */
	switch (rc)	{
	    case 0: break;
	    case -1: 
		glexec_log(GLEXEC_LOG_ERROR,
		    "Error: error creating dir for LCAS logfile '%s'.\n",
		    lcas_log_name);
		glexec_exit(GLEXEC_LCAS_INIT_FAIL, GLEXEC_NO_CHILD_EXIT_CODE);
	    case -2:
		glexec_log(GLEXEC_LOG_ERROR,
		    "Error: I/O error during opening LCAS logfile: %s.\n",
		    strerror(errno));
		glexec_exit(GLEXEC_LCAS_INIT_FAIL, GLEXEC_NO_CHILD_EXIT_CODE);
	    case -3:
		glexec_log(GLEXEC_LOG_WARNING,
		    "WARNING: Permissions for LCAS logfile are wrong " \
		    "(should be: user root, " \
		    "permissions: file at most 750, dir at most 755). "\
		    "In next version this will be a fatal error.\n");
		break;
	    case -4:
		glexec_exit(GLEXEC_OUT_OF_MEMORY, GLEXEC_NO_CHILD_EXIT_CODE);
	    case -5:
		glexec_log(GLEXEC_LOG_ERROR,
		    "Error: LCAS logfile '%s' is not an absolute path.\n",
		    lcas_log_name);
		glexec_exit(GLEXEC_LCAS_INIT_FAIL, GLEXEC_NO_CHILD_EXIT_CODE);
	    case -6:
		glexec_log(GLEXEC_LOG_ERROR,
		    "Error: LCAS logfile is no longer the one that was opened!.\n");
		glexec_exit(GLEXEC_LCAS_INIT_FAIL, GLEXEC_NO_CHILD_EXIT_CODE);
	    case -7:
		glexec_log(GLEXEC_LOG_ERROR,
		    "Error: Specified LCAS logfile is not a regular file.\n");
		glexec_exit(GLEXEC_LCAS_INIT_FAIL, GLEXEC_NO_CHILD_EXIT_CODE);
	    case -8:
	    default:
		glexec_exit(GLEXEC_LCAS_INIT_FAIL, GLEXEC_NO_CHILD_EXIT_CODE);
	}

	/* Log the log filename */
	glexec_log(GLEXEC_LOG_DEBUG,
	    "LCAS will be initialised with log file: %s\n", lcas_log_name);
    }

    return lcas_log;
}

==> glexec_0.9.11.orig.tar.gz: glexec-0.9.11/src/glexec_lcas.c <==
/**
 * Copyright (c) Members of the EGEE Collaboration. 2010. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 * Authors: Oscar Koeroo, Mischa Sall\'e
 *      NIKHEF Amsterdam, the Netherlands
 *      <grid-mw-security@nikhef.nl>  
 */

/*
 * glexec_lcas provides the functionality to dynamically open lcas and its API
 */

#define _XOPEN_SOURCE	600

#include "glexec.h"
#include "glexec_conf.h"
#include "glexec_log.h"
#include "glexec_environ.h"
#include "glexec_util.h"
#include "glexec_lcas.h"

#include "fileutil/fileutil.h"

#include <dlfcn.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/stat.h> /* For GLEXEC_LOG_FILEMASK / GLEXEC_LOG_DIRMASK */
#include <unistd.h>

/*************************************************************************/
/* TYPEDEFS			                                         */
/*************************************************************************/

/* Either include lcas_types.h or just takeover the definition of the
 * lcas_request_t
 */
#if 0
#include "lcas_types.h"
#else
typedef char *lcas_request_t;
#endif

/* Struct containing all the lcas functions, will be filled using dlsym() */
typedef struct {
    void *handle; /* handle to the actual LCAS library */
    int (*lcas_init)(FILE*);
    int (*lcas_pem)(char*,lcas_request_t);
    int (*lcas_term)(void);
} lcas_library_t;

/*************************************************************************/
/* INTERNAL FUNCTION PROTOTYPES                                          */
/*************************************************************************/

/*
 * Loads the LCAS library and its functions using dlopen() and dlsym()
 * returns 0 on success and -1 on error.
 */
static int glexec_lcas_load(const char *libdir, lcas_library_t *mylcas_handle);

/*
 * Closes the LCAS library.
 * returns 0 on success and -1 on error.
 */
static int glexec_lcas_close(lcas_library_t *mylcas_handle);

/*
 * Setup LCAS environment variables
 * set libdir (might be "") where to find lcas.so
 */
static void glexec_lcas_env_setup(const char **libdir);

/*
 * Open LCAS logging.
 * returns pointer to glexec_lcas_log file (might be NULL)
 */
static FILE *glexec_lcas_log_setup(void);

/*************************************************************************/
/* FUNCTIONS			                                         */
/*************************************************************************/

/*
 * Setup, run and close LCAS for given rsl.
 * returns GLEXEC_SUCCESS when successful
 */
int glexec_lcas(char *pem_string, char *rsl)   {
    const char *mylcas_libdir;
    FILE *mylcas_log;
    lcas_library_t mylcas_handle;
    lcas_request_t request=(lcas_request_t)rsl;
    int rc;

    /* Setup LCAS environment */
    glexec_lcas_env_setup(&mylcas_libdir);

    /* Load LCAS library */
    if (glexec_lcas_load(mylcas_libdir,&mylcas_handle))
	return GLEXEC_LCAS_INIT_FAIL;

    /* Setup LCAS log */
    mylcas_log=glexec_lcas_log_setup();
    
    /* Init will return 0 on success or otherwise on failure, note that
     * mylcas_log can be NULL in which case syslog is used. */
    if (mylcas_handle.lcas_init(mylcas_log))    {
	rc=GLEXEC_LCAS_INIT_FAIL;
	goto lcas_finalize_noterm;
    }

    glexec_log(GLEXEC_LOG_DEBUG, "Initialisation of LCAS has finished.\n");

    /* LCAS AuthZ call */
    rc=(mylcas_handle.lcas_pem(pem_string, request)==0 
	    ? GLEXEC_SUCCESS
	    : GLEXEC_LCAS_FAILED);

    /* terminate LCAS */
    mylcas_handle.lcas_term();

lcas_finalize_noterm:
    /* close the library */
    glexec_lcas_close(&mylcas_handle);

    /* Close the log file only if one is actually open (like in syslog usage
     * mode) */
    if (mylcas_log) {
	if (fclose(mylcas_log))
	    glexec_log(GLEXEC_LOG_ERROR,
		    "Error while closing lcas log file: %s\n",
		    strerror(errno));
	mylcas_log = NULL;
    }

    /* Now check whether LCAS succeeded */
    if (rc==GLEXEC_SUCCESS)
	glexec_log(GLEXEC_LOG_DEBUG, "LCAS is finished\n");
    
    return rc;
}

/*
 * Print the known LCAS defines to file out.
 */
void glexec_lcas_defines(FILE *out)	{
#ifdef LCAS_DB_FILE
    fprintf(out," -D LCAS_DB_FILE=\"%s\"\n", LCAS_DB_FILE);
#endif
#ifdef LCAS_LOG_FILE
    fprintf(out," -D LCAS_LOG_FILE=\"%s\"\n", LCAS_LOG_FILE);
#endif
#ifdef LCAS_DEBUG_LEVEL
    fprintf(out," -D LCAS_DEBUG_LEVEL=\"%s\"\n", LCAS_DEBUG_LEVEL);
#endif
#ifdef LCAS_MODULEDIR_SFX
    fprintf(out," -D LCAS_MODULEDIR_SFX=\"%s\"\n", LCAS_MODULEDIR_SFX);
#endif
#ifdef LCAS_LIBDIR
    fprintf(out," -D LCAS_LIBDIR=\"%s\"\n", LCAS_LIBDIR);
#endif
}

/*************************************************************************/
/* INTERNAL FUNCTIONS			                                 */
/*************************************************************************/

/*
 * Loads the LCAS library and its functions using dlopen() and dlsym()
 * returns 0 on success and -1 on error.
 */
static int glexec_lcas_load(const char *libdir, lcas_library_t *mylcas)  {
    char *error=dlerror(); /* proper way to check dlsym() failed is using
			      dlerror(), see dlsym(3) */
    const char *mylcas_so="liblcas" LIBSUFF;
    char mylcas_lib_path[AP_MAXPATH];
    const char *mylcas_lib;
    int rc_sn;

    /* prefix lcas library name with libdir when needed */
    if (strlen(libdir)>0)   {
	rc_sn=snprintf(mylcas_lib_path,(size_t)AP_MAXPATH,"%s/%s",libdir,mylcas_so);
	if (rc_sn<0)	{
	    glexec_log(GLEXEC_LOG_ERROR,"Error creating path for liblcas\n");
	    return -1;
	}
	if (rc_sn>=AP_MAXPATH)    {
	    glexec_log(GLEXEC_LOG_ERROR,"Full path %s/%s is too long\n",
		    libdir,mylcas_so);
	    return -1;
	}
	mylcas_lib=mylcas_lib_path;
    } else
	mylcas_lib=mylcas_so;

    /* Load the library: note need RTLD_GLOBAL to provide its symbols to the
     * plugins! This also means they stay around after a dlclose() ! */
    glexec_log(GLEXEC_LOG_DEBUG, "Opening LCAS lib: %s\n",mylcas_lib);
    if ( (mylcas->handle=
		dlopen(mylcas_lib,RTLD_NOW | RTLD_GLOBAL))==NULL )	{
	glexec_log(GLEXEC_LOG_ERROR,
		   "Error opening LCAS lib: %s\n",dlerror());
	return -1;
    }

    /* Load necessary basic LCAS functions */
    glexec_log(GLEXEC_LOG_DEBUG, "Loading LCAS symbols.\n");
    mylcas->lcas_init=(int (*)(FILE*))dlsym(mylcas->handle,"lcas_init");
    if ((error=dlerror())!=NULL)
	goto lcas_dlopen_failed;
    mylcas->lcas_pem=
	(int (*)(char*,lcas_request_t))dlsym(mylcas->handle,"lcas_pem");
    if ((error=dlerror())!=NULL)
	goto lcas_dlopen_failed;
    mylcas->lcas_term=(int (*)(void))dlsym(mylcas->handle,"lcas_term");
    if ((error=dlerror())!=NULL)
	goto lcas_dlopen_failed;
    return 0;
    
lcas_dlopen_failed:
    glexec_log(GLEXEC_LOG_ERROR,"Error loading LCAS function: %s\n",error);
    dlclose(mylcas->handle);
    mylcas->handle=NULL;
    return -1;
}

/*
 * Closes the LCAS library.
 * returns 0 on success and -1 on error.
 */
static int glexec_lcas_close(lcas_library_t *mylcas)	{
    int rc;
    glexec_log(GLEXEC_LOG_DEBUG,"Closing LCAS library.\n");
    if ((rc=dlclose(mylcas->handle)))
	glexec_log(GLEXEC_LOG_ERROR,"Error closing LCAS library: %s\n",
		dlerror());
    mylcas->handle=NULL;
    return (rc ? -1 : 0);
}

/*
 * Setup LCAS environment variables
 * set libdir (might be "") where to find lcas.so
 */
static void glexec_lcas_env_setup(const char **mylcas_libdir) {
    const char *conflibdir,*libdir;
    const char *moddir_suffix;
    char modules_dir[AP_MAXPATH];
    char *jobrepoid;
    int rc_sn;

    /* Get jobrepo ID for LCAS_LOG_STRING */
    jobrepoid=getenv("JOB_REPOSITORY_ID");
    setenv("LCAS_LOG_STRING", (jobrepoid ? jobrepoid : "NULL"), 1);

    /* Get LCAS variables */
    glexec_setenv_regex("LCAS_DEBUG_LEVEL",
	glexec_conf_value_default("glexec:lcas_debug_level",LCAS_DEBUG_LEVEL),
	"^[0-5]$");
    glexec_setenv_regex("LCAS_DB_FILE",
	glexec_conf_value_default("glexec:lcas_db_file",LCAS_DB_FILE),
	"\\/(.*\\/)+[^/]*");

    /* Setup LCAS libdir, will be used for dlopen-ing LCAS */

    /* First see if there is a config file value */
    conflibdir=glexec_conf_value("glexec:lcas_libdir");
    if ( conflibdir )	{
	if (conflibdir[0]=='\0' || glexec_dir_is_abs(conflibdir))
	    libdir=conflibdir;
	else	{
	    glexec_log(GLEXEC_LOG_WARNING,
		"Warning: ignoring lcas_libdir config option: "
		"\"%s\" is not an absolute path to a valid directory.\n",
		conflibdir);
	    libdir=LCAS_LIBDIR;
	}
    } else
	libdir=LCAS_LIBDIR;
    
    /* If conflibdir didn't do the trick, try LCAS_LIBDIR */
    if (libdir!=conflibdir && libdir[0]!='\0' && !glexec_dir_is_abs(libdir))   {
	glexec_log(GLEXEC_LOG_WARNING,
	    "Warning: ignoring built-in LCAS_LIBDIR: "
	    "\"%s\" is not an absolute path to a valid directory.\n",
	    libdir);
	libdir="";
    }
    *mylcas_libdir=libdir;

    /* If it's non-empty, also setup LCAS_MODULES_DIR. */
    if ( libdir[0]!='\0')    { /* No need for strlen... */
	glexec_log(GLEXEC_LOG_DEBUG,"Will look for LCAS in '%s'\n",libdir);
	
	/* Get suffix where to look for LCAS modules, used to be /modules,
	 * changed to /lcas to satisfy Fedora guidelines */
	moddir_suffix=glexec_conf_value_default("glexec:lcas_moduledir_sfx",
						LCAS_MODULEDIR_SFX);
    
	rc_sn=snprintf(modules_dir,(size_t)AP_MAXPATH,"%s%s",
		libdir, moddir_suffix);
	if (rc_sn<0)	{
	    glexec_log(GLEXEC_LOG_WARNING,
		"Cannot create modulespath, not setting LCAS_MODULES_DIR\n");
	} else if (rc_sn >= AP_MAXPATH)  {
	    glexec_log(GLEXEC_LOG_WARNING,
		"Full modulespath '%s%s' would be too long, " \
		"not setting LCAS_MODULES_DIR\n",
		libdir,moddir_suffix);
	} else	{
	    if (!glexec_dir_is_abs(modules_dir))
		glexec_log(GLEXEC_LOG_WARNING,
		    "Warning: modulespath '%s' does not point to absolute "
		    "directory, not setting LCAS_MODULES_DIR\n",modules_dir);
	    else    {
		glexec_log(GLEXEC_LOG_DEBUG,
			"Setting LCAS_MODULES_DIR to '%s'\n", modules_dir);
		setenv("LCAS_MODULES_DIR", modules_dir, 1);
	    }
	}
    }
}

/*
 * Open LCAS logging.
 * returns pointer to glexec_lcas_log file (might be NULL)
 */
static FILE *glexec_lcas_log_setup(void) {
    const mode_t file_mask=GLEXEC_LOG_FILEMASK; /* File permissions */
    const mode_t dir_mask=GLEXEC_LOG_DIRMASK; /* Dir permissions */
    int rc;
    gid_t oldgid=getegid(),mylcas_loggid;
    char *mylcas_log_name;
    FILE *mylcas_log=NULL;

    /* Optionally open logfile */
    if (glexec_get_log_to_syslog()) { /* syslog */
	/*  Make sure the environment variables are never set when glexec has to
	 *  use syslog.*/
	unsetenv("LCAS_LOG_FILE");
    } else { /* logfile */
	glexec_setenv_regex("LCAS_LOG_FILE",
	    glexec_conf_value_default("glexec:lcas_log_file",LCAS_LOG_FILE),
		"\\/(.*\\/)+[^/]*");
	if ( (mylcas_log_name = getenv("LCAS_LOG_FILE"))==NULL )	{
	    glexec_log(GLEXEC_LOG_ERROR,
		"Log file for LCAS could not be opened: "\
		"No log file name was specified.\n");
	    glexec_exit (GLEXEC_LCAS_INIT_FAIL, GLEXEC_NO_CHILD_EXIT_CODE);
	}
	/* Get logging GID as set by glexec */
	mylcas_loggid=glexec_get_loggid();
	/* Set the effective gid, so that file and dir will be created with the
	 * right ownership */
	if (setegid(mylcas_loggid))
	    glexec_log(GLEXEC_LOG_WARNING,
		"Setting effective gid to %d failed.\n", (int)mylcas_loggid);
	/* Now try to open the logfile with file permission file_mask and directory
	 * permissions dir_mask. Don't force it to be trusted. */
	rc=cgul_open_logfile(mylcas_log_name, file_mask, dir_mask, &mylcas_log, 0);
	/* reset effective gid */
	if (setegid(oldgid))
	    glexec_log(GLEXEC_LOG_WARNING,
		"Setting effective gid to %d failed.\n", (int)oldgid);
	/* Parse return code of cgul_open_logfile */
	switch (rc)	{
	    case 0: break;
	    case -1: 
		glexec_log(GLEXEC_LOG_ERROR,
		    "Error: error creating dir for LCAS logfile '%s'.\n",
		    mylcas_log_name);
		glexec_exit(GLEXEC_LCAS_INIT_FAIL, GLEXEC_NO_CHILD_EXIT_CODE);
	    case -2:
		glexec_log(GLEXEC_LOG_ERROR,
		    "Error: I/O error during opening LCAS logfile: %s.\n",
		    strerror(errno));
		glexec_exit(GLEXEC_LCAS_INIT_FAIL, GLEXEC_NO_CHILD_EXIT_CODE);
	    case -3:
		glexec_log(GLEXEC_LOG_WARNING,
		    "Warning: Permissions for LCAS logfile are wrong " \
		    "(should be: user root, " \
		    "permissions: file at most 750, dir at most 755). "\
		    "In next version this will be a fatal error.\n");
		break;
	    case -4:
		glexec_exit(GLEXEC_OUT_OF_MEMORY, GLEXEC_NO_CHILD_EXIT_CODE);
	    case -5:
		glexec_log(GLEXEC_LOG_ERROR,
		    "Error: LCAS logfile '%s' is not an absolute path.\n",
		    mylcas_log_name);
		glexec_exit(GLEXEC_LCAS_INIT_FAIL, GLEXEC_NO_CHILD_EXIT_CODE);
	    case -6:
		glexec_log(GLEXEC_LOG_ERROR,
		    "Error: LCAS logfile is no longer the one that was opened!.\n");
		glexec_exit(GLEXEC_LCAS_INIT_FAIL, GLEXEC_NO_CHILD_EXIT_CODE);
	    case -7:
		glexec_log(GLEXEC_LOG_ERROR,
		    "Error: Specified LCAS logfile is not a regular file.\n");
		glexec_exit(GLEXEC_LCAS_INIT_FAIL, GLEXEC_NO_CHILD_EXIT_CODE);
	    case -8:
	    default:
		glexec_exit(GLEXEC_LCAS_INIT_FAIL, GLEXEC_NO_CHILD_EXIT_CODE);
	}

	/* Log the log filename */
	glexec_log(GLEXEC_LOG_DEBUG,
	    "LCAS will be initialised with log file: %s\n", mylcas_log_name);
    }

    return mylcas_log;
}
